from pyboon.qs import *

def test_qs():
    assert 'k=v&v=%E7%BF%BB%E8%AF%91' == stringify({"k":"v","v":"翻译"})
    assert 'k=v&v=翻译' == stringify({"k":"v","v":"翻译"},u=True)
    assert {"k":"v","v":"k"} == parse('k=v&v=k')


import os,json
from  openpyxl import  Workbook ,load_workbook
from pyboon import file as f
from rich.console import Console

def csv2xl(csvfile):
    wb = Workbook()
    ws = wb.active

    txt = f.read(csvfile)

    for line in txt.split("\n"):
        print(line)
        ws.append(line.split(","))


    wb.save(csvfile.replace(".txt",".xlsx"))

def xl2csv(xlfile):
    wb = load_workbook(xlfile)
    ws = wb[wb.sheetnames[0]]

    lines = []
    for row in  ws.iter_rows():
        values = [cell.value for cell in  row]
        if values[0]:
            lines.append(",".join(values))
    
    f.write(os.path.join(os.getcwd(), xlfile.replace(".xlsx",".txt")),"\n".join(lines))

def to_json(d):
    return json.dumps(d,ensure_ascii=False)

def to_json_array(l):
    return [to_json(x) for x in l]

def to_md_line(d):
    return "|\t"+"|\t".join([str(v) for _,v in d.items()])+"|"

def to_md(l,head=[],show=[]):
    if not l or not l[0]:
        return ""
    if not head:
        head = "|\t"+"|\t".join([k for k,_ in l[0].items()])+"|"
    if not show:
        show = head
    splitline = "|"+"\t|".join(["---:" for k,_ in l[0].items()])+"\t|"
    lines = [show,splitline]
    for row in l:
        line = to_md_line(row)
        lines.append(line)
    return "\n".join(lines)

def to_csv_line(d):
    return ",".join([str(v) for _,v in d.items()])

def to_csv(l,head=[],show=[]):
    if not l or not l[0]:
        return ""
    if not head:
        head = ",".join([k for k,_ in l[0].items()])
    if not show:
        show = head
    lines = [show]
    for row in l:
        line = to_csv_line(row)
        lines.append(line)
    return "\n".join(lines)

def to_xlsx(l,head=[],show=[]):
    wb = Workbook()
    ws = wb.active

    for line in to_csv(l).split("\n"):
        print(line)
        ws.append(line.split(","))

    wb.save("a.xlsx")

if __name__ == '__main__':
    console = Console()
    console.print(to_csv([{"a":1,"b":"c"},{"a":2,"b":"c"}]))
    console.print(to_xlsx([{"a":1,"b":"c"},{"a":2,"b":"c"}]))
    console.print(to_md([{"a":1,"b":"c"},{"a":2,"b":"c"}]))


from pyboon.convert import *

def test_convert():
    dts = "2021-12-12 09:10:11"
    dt = datetime.datetime(2021, 12, 12, 9, 10, 11)
    assert dt.timetuple() == time.strptime(dts, "%Y-%m-%d %H:%M:%S")
    assert dts == time.strftime("%Y-%m-%d %H:%M:%S",dt.timetuple())
    assert dt == to_datetime(dts)
    assert dts == to_datetime_str(dt)
    assert 1 == to_int('1')
    assert 1.0 == to_float('1')
    assert '1' == to_str(1)
    assert '1.0' == to_str(1.0)
    assert {"k":"v","v":"汉字"} == to_dict('{"k":"v","v":"汉字"}')
    assert '{"k": "v", "v": "汉字"}' == to_json({"k":"v","v":"汉字"})


from pyboon.x import *

def test_to_xxx():
    assert 'hello new world' == to_seq("HelloNew_world")
    assert 'HelloNewWorld'   == to_pascal("HelloNew_world")
    assert 'helloNewWorld'   == to_camel("HelloNew_world")
    assert 'hello_new_world' == to_underscore("HelloNew_world")
    assert 'hello-new-world' == to_minus("HelloNew_world")


from pyboon.chain import Chain
from pyboon import __

def test_chain():
    items = [1,2,3]
    chain = Chain(items)
    assert [1,2,3] == chain.items
    assert [1,2,3] == chain.to_list()
    assert 1 == chain.first()
    assert 2 == chain.map(lambda x:x+1).first()
    assert 2 == __.chain(items).map(lambda x:x+1).first()


from pyboon.rest import *


def test_rest():
    assert {"a": '1'} == get("http://httpbin.org/get", {"a": '1'})['args']
    assert {"a": '1'} == post("http://httpbin.org/post", {"a": '1'})['json']
    assert {"a": '1'} == json("http://httpbin.org/post", {"a": '1'})['json']
    assert {"a": '1'} == form("http://httpbin.org/post", {"a": '1'})['form']



import pymysql
import dataset
import re

pymysql.install_as_MySQLdb()


def get_db(url):
    return dataset.connect(url, engine_kwargs={'encoding': 'utf-8'})


def qb(l,sql):
    ret = re.search(r"SELECT (.*) FROM", sql)
    _, v = ret.group(0), ret.group(1)
    fields = [field.strip() for field in v.split(",")]
    print(fields)
    result = []
    for item in l:
        n = {}
        for field in fields:
            if field in item:
                n[field] = item[field]
            else:
                print(field,"not in item")
        result.append(n)
    print(result)
    return result

def qb_one(l,sql):
    ret = re.search(r"SELECT (.*) FROM", sql)
    _, v = ret.group(0), ret.group(1)
    fields = [field.strip() for field in v.split(",")]
    print(fields)
    result = []
    for item in l:
        n = {}
        for field in fields:
            if field in item:
                n[field] = item[field]
            else:
                print(field,"not in item")
        result.append(n)
    print(result)
    return result[0] if result else None


if __name__ == '__main__':
    qb([{"a": 1, "b": 2, "c": 3, "d": 4, }],"SELECT a,b FROM tab")



import datetime
import json
import time

def to_int(p):
    return int(p)

def to_str(p):
    return str(p)

def to_float(p):
    return float(p)

def to_datetime(s):
    return datetime.datetime.strptime(s, "%Y-%m-%d %H:%M:%S")

def to_datetime_str(dt):
    return time.strftime("%Y-%m-%d %H:%M:%S",dt.timetuple())

def to_json(d):
    return json.dumps(d,ensure_ascii=False)

def to_dict(s):
    return json.loads(s)



from pyboon import _


def test_underscore():
    items = [1, 2, 3]
    assert [2, 3, 4] == _.map2(items, lambda x: x+1)
    assert [2, 3] == _.filter2(items, lambda x: x > 1)
    assert 2 == _.find(items, lambda x: x > 1)
    assert 6 == _.reduce2(items, lambda x, y: x+y)
    assert 3 == _.size(items)
    assert 6 == _.size("123456")
    assert [0, 1, 2] == _.range2(3)
    assert [1, 2, 3, 4] == _.range2(1, 5)
    assert [1, 3] == _.range2(1, 5, 2)
    assert [1, -1, -3] == _.range2(1, -5, -2)
    assert "abcabc" == _.repeat("abc")
    assert "abcabcabcabcabcabc" == _.repeat("abc", 6)
    assert "000001" == "1".zfill(6)
    assert "000001" == _.pad("1",6)



from urllib.parse import urlencode,parse_qs,unquote

def stringify(d,u=False):
    qs = urlencode(d)
    if u:
        qs = unquote(qs)
    return qs

def parse(url):
    d = dict( (k, v if len(v)>1 else v[0] )
           for k, v in parse_qs(url).items() )
    return d


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import codecs
import chevron

CURRENT_DIR = os.path.split(os.path.realpath(__file__))[0]


def current_src_dir():
    return CURRENT_DIR


def read(rpath):
    with codecs.open(rpath, 'r', encoding='utf-8') as f:
        content = f.read()
    return content


def write(rpath, content):
    dir, _ = os.path.split(rpath)
    if not os.path.exists(dir):
        os.makedirs(dir)
    with codecs.open(rpath, 'w', encoding='utf-8') as f:
        f.write(content)


def ms(args):
    return chevron.render(**args, def_ldel='{{', def_rdel='}}')


def tpl(path_template, data):
    t = os.path.join(current_src_dir(), "templates", path_template)
    with codecs.open(t, "r", encoding='utf-8') as f:
        content_template = f.read()
    path_args = {'template': path_template, 'data': data}
    content_args = {'template': content_template, 'data': data}
    path, content = ms(path_args), ms(content_args)
    rpath = os.path.join(current_src_dir(), path.replace(".mst", ""))
    write(rpath, content)
    return rpath, content


def rm(path_template, data):
    path_args = {'template': path_template, 'data': data}
    path = ms(path_args)
    rpath = os.path.join(current_src_dir(), path.replace(".mst", ""))
    rmpath(rpath)


def rmdir(path_template, data):
    path_args = {'template': path_template, 'data': data}
    path = ms(path_args)
    rpath = os.path.join(current_src_dir(), path)
    shutil.rmtree(rpath)


def rmpath(rpath):
    if os.path.exists(rpath):  # 如果文件存在
        # 删除文件，可使用以下两种方法。
        os.remove(rpath)
        # os.unlink(rpath) #正在使用会报错
    else:
        print('no such file:%s' % rpath)  # 则返回文件不存在


def _rmline(rpath, rline):
    content = read(rpath)
    content = content.replace(rline, '')
    write(rpath, content)
    print("_rmline", rpath, rline)


def rmline(path, line_template, data):
    line_args = {'template': line_template, 'data': data}
    rline = ms(line_args)
    rpath = os.path.join(current_src_dir(), path.replace(".mst", ""))
    _rmline(rpath, rline)


def _insert_before(rpath, xline, rline):
    content = read(rpath)
    content = content.replace(xline, rline+"\n"+xline)
    write(rpath, content)
    print("_insert_before", rpath, rline)


def insert_before(path, xline, line_template, data):
    line_args = {'template': line_template, 'data': data}
    rline = ms(line_args)
    rpath = os.path.join(current_src_dir(), path.replace(".mst", ""))
    _insert_before(rpath, xline, rline)



import requests

headers = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.104 Safari/537.36"
}


def get(url, data, headers=headers):
    return requests.get(url, params=data, headers=headers).json()


def post(url, data, headers=headers):
    return json(url, data, headers=headers)


def json(url, data, headers=headers):
    return requests.post(url, json=data, headers=headers).json()


def form(url, data, headers=headers):
    return requests.post(url, data=data, headers=headers).json()



from pyboon.sql import *

def test_sql():
    db = get_db('mysql://root:aFM4JrZIDgpGiqJn@localhost:3306/hello?charset=UTF8MB4')
    assert 1 == db.query('select 1 from dual').next()['1']

def test_qb():
    assert [{'a': 1, 'b': 2}] == qb([{"a":1,"b":2,"c":3,"d":4,}],"SELECT a , b,e FROM tab")
    assert {'a': 1, 'b': 2} == qb_one([{"a":1,"b":2,"c":3,"d":4,}],"SELECT a,b,e FROM tab")


import datetime
import json
import time
import requests
import random
from hashlib import md5
from rich.console import Console
from rich.syntax import Syntax


def highlight(code, language="python"):
    syntax = Syntax(code, language)
    console = Console()
    console.print(syntax)


def to_dict(s):
    return json.loads(s, encoding="UTF8")


def to_json(d):
    return json.dumps(d, ensure_ascii=False)


def now_time_str():
    return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")


def delta_time_str(**delta):
    return (datetime.datetime.now() + datetime.timedelta(**delta)).strftime("%Y-%m-%d %H:%M:%S")


def to_time_str(t):
    return time.strftime("%Y-%m-%d %H:%M:%S", t)


def to_local_time(s):
    return time.strptime(s, "%Y-%m-%d %H:%M:%S")


def to_time_ms(s):
    return int(time.mktime(to_local_time(s)) * 1000)


def now_ms():
    return int(time.time() * 1000)


def to_seq(s):
    text = s.strip()
    lst = []
    for index, char in enumerate(text):
        if char.isupper() and index != 0:
            lst.append("_")
        lst.append(char)

    return "".join(lst).lower().replace("__", " ").replace("_", " ")


def to_pascal(s):
    us = to_seq(s)
    tus = us.title().replace(" ", "")
    return tus


def to_camel(s):
    us = to_seq(s)
    tus = us.title().replace(" ", "")
    return tus[:1].lower()+tus[1:]


def to_underscore(s):
    us = to_seq(s)
    return us.replace(" ", "_")


def to_minus(s):
    us = to_seq(s)
    return us.replace(" ", "-")


def translate(query):
    url = 'http://fanyi.youdao.com/translate'
    payload = {'doctype': "json", 'type': "AUTO", 'i': query}
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.104 Safari/537.36"}
    r = requests.get(url, params=payload, headers=headers)
    result = r.json()["translateResult"][0][0]["tgt"]
    return result


if __name__ == '__main__':
    highlight('''import rich''')
    print(to_seq("HelloNew_world"))
    print(to_pascal("HelloNew_world"))
    print(to_camel("HelloNew_world"))
    print(to_underscore("HelloNew_world"))
    print(to_minus("HelloNew_world"))




from functools import reduce


def map2(items, func):
    items = map(func, items)
    return list(items)


def filter2(items, func):
    items = filter(func, items)
    return list(items)


def find(items, func):
    items = filter(func, items)
    items = list(items)
    return items[0] if items else None


def reduce2(items, func):
    return reduce(func, items)


def range2(*into):
    if isinstance(into, list) or isinstance(into, tuple):
        return list(range(*into))
    return list(range(into))


def size(into):
    return len(into)


def trim(s):
    return s.strip()


def repeat(s, n=2):
    return s*n


def pad(s, n=0, c='0'):
    if n <= len(s):
        return s
    return c*(n-len(s))+s





class Chain:
    items = []

    def __init__(self,into):
        if isinstance(into,list):
            self.items = into

    @classmethod
    def chain(cls,into):
        return Chain(into)
    


    def map(self,func):
        items = map(func,self.items)
        return Chain(list(items))

    def to_list(self):
        return self.items

    def first(self):
        return self.items[0] if self.items else None